package com.wjholden.nmap;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import android.content.Context;
import android.content.res.Resources;
import android.os.AsyncTask;
import android.os.Message;

/**
 * Verifies the installation of Nmap binaries asynchronously.
 * Checks whether root permissions are available or not (Nmap.canGetRoot).
 * Creates the binary directory (Nmap.bindir).
 * 
 * TODO: Use md5 sums of files to ensure integrity. Maybe better as a user-driven process?
 * 
 * @author William John Holden (wjholden@gmail.com)
 * @version 0.4.6
 * @since 0.4.6 COMPLETE REWRITE
 */
public class NmapInstall extends AsyncTask <Object, Void, Void> implements NmapConstants
{
	/**
	 * List of filenames corresponding to those resources made available in requiredResources[].
	 */
	private final String filenames[] = { "ndiff", "nping", "nmap-services", "nmap-mac-prefixes" };
	
	/**
	 * The name is somewhat misleading now, but these are a few files that should be 
	 * installed during this process.  The list is not all-inclusive (not R.raw.nmap absent)
	 * because some files are so large they had to be split, therefore they couldn't be 
	 * installed in this neat interative process.
	 */
	private final int requiredResources[] = { R.raw.ndiff, R.raw.nping, R.raw.nmap_services, R.raw.nmap_mac_prefixes };
	
	/**
	 * Once you have the application Context, use that to establish a Resources object that can be used to open
	 * the R.raw.* files.
	 */
	private transient Resources resources;
	
	private boolean forceRestart;
	
	protected void onPostExecute(final Void myVoid)
	{
		NmapMain.pd.dismiss();
		NmapMain.installVerified = true;
		
		if (forceRestart)
		{
			NmapMain.handler.sendEmptyMessage(FORCE_RESTART);
		}
	}

	@Override
	protected Void doInBackground(final Object... params)
	{
		Context context;
		
		if (params[0] == null)
		{
			Message msg = Message.obtain();
			msg.obj = "Context is null (use new VerifyInstallation(Context)).";
			msg.what = INSTALL_ERROR;
			NmapMain.handler.sendMessage(msg);
			return null;
		}
		context = (Context) params[0];
		
		forceRestart = params[1].equals("restart");
		
		resources = context.getResources();
		NmapMain.bindir = NmapUtilities.findBinDir(NmapUtilities.getDataDirectory(context.getApplicationContext()));

		try
		{
			writeNmap();
			writeNcat();
			for (int k = 0; k < requiredResources.length; k++)
			{
				writeOthers(filenames[k], requiredResources[k]);
			}
			setPermissions();
		}
		catch (IOException e)
		{
			Message msg = Message.obtain();
			msg.obj = "Installation error: " + e.toString();
			msg.what = INSTALL_ERROR;
			NmapMain.handler.sendMessage(msg);
		} catch (InterruptedException e) {
			Message msg = Message.obtain();
			msg.obj = e.toString();
			msg.what = INSTALL_ERROR;
			NmapMain.handler.sendMessage(msg);
		}
		finally
		{
			// if you make to here that means there's absolutely no error so blast that message up and keep going.
			NmapMain.handler.sendEmptyMessage(INSTALL_GOOD);
			NmapMain.ne.log("Installation successful.");
		}
		return null;
	}
	
	private void createNewFileAndDeleteExisting(final String path)
	{
		final File myFile = new File (path);
		if (myFile.exists())
		{
			NmapMain.ne.log(path + " already exists. Deleting...");
			if (myFile.delete())
			{
				NmapMain.ne.log("\tdeleted.");
			}
			else
			{
				NmapMain.ne.log("\tunable to delete.");
			}
		}
	}
	
	private void writeNmap() throws IOException
	{
		byte[] buf = new byte[BUFFER_SIZE];
		createNewFileAndDeleteExisting(NmapMain.bindir + "/nmap");
		
		InputStream in_nmap_a = resources.openRawResource(R.raw.nmap_a);
		InputStream in_nmap_b = resources.openRawResource(R.raw.nmap_b);
		InputStream in_nmap_c = resources.openRawResource(R.raw.nmap_c);
		OutputStream out_nmap = new FileOutputStream(NmapMain.bindir + "/nmap");
		while (in_nmap_a.read(buf) > 0)
		{
			out_nmap.write(buf);
		}
		in_nmap_a.close();
		while (in_nmap_b.read(buf) > 0)
		{
			out_nmap.write(buf);
		}
		in_nmap_b.close();
		while (in_nmap_c.read(buf) > 0)
		{
			out_nmap.write(buf);
		}
		in_nmap_c.close();
		out_nmap.close();
	}
	
	private void writeNcat() throws IOException
	{
		byte[] buf = new byte[BUFFER_SIZE];
		createNewFileAndDeleteExisting(NmapMain.bindir + "/ncat");
		
		InputStream in_ncat_a = resources.openRawResource(R.raw.ncat_a);
		InputStream in_ncat_b = resources.openRawResource(R.raw.ncat_b);
		OutputStream out = new FileOutputStream(NmapMain.bindir + "/ncat");
		while (in_ncat_a.read(buf) > 0)
		{
			out.write(buf);
		}
		in_ncat_a.close();
		while (in_ncat_b.read(buf) > 0)
		{
			out.write(buf);
		}
		in_ncat_b.close();
		out.close();
	}
	
	private void writeOthers(final String filename, final int resource) throws IOException
	{
		byte[] buf = new byte[BUFFER_SIZE];
		createNewFileAndDeleteExisting(NmapMain.bindir + "/" + filename);
		
		InputStream in1 = resources.openRawResource(resource);
		OutputStream out = new FileOutputStream(NmapMain.bindir + "/" + filename);
		while (in1.read(buf) > 0)
		{
			out.write(buf);
		}
		in1.close();
		out.close();
	}
	
	private void setPermissions() throws IOException, InterruptedException
	{
		String installationResults;
		//java.io.File file_nmap = new java.io.File(Nmap.bindir + "/nmap");
		//file_nmap.setExecutable(true); // not implemented? WTF?!!! RTFM it's API9+ only.
		
		Process process = NmapMain.canGetRoot ? Runtime.getRuntime().exec("su") :  Runtime.getRuntime().exec("sh");
		DataOutputStream os = new DataOutputStream(process.getOutputStream());
		BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()), BUFFER_SIZE);
		BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream()), BUFFER_SIZE);
		os.writeBytes("cd " + NmapMain.bindir + "\n");
		/*if (Nmap.canGetRoot)
		{
			os.writeBytes("chown root.root nmap\n");
			Nmap.ne.Log("chown root.root nmap");
			os.writeBytes("chown root.root nping\n");
			Nmap.ne.Log("chown root.root nping");
			os.writeBytes("chown root.root ncat\n");
			Nmap.ne.Log("chown root.root ncat");
		}
		os.writeBytes("chmod 555 nmap\n");
		Nmap.ne.Log("chmod 555 nmap");
		os.writeBytes("chmod 555 nping\n");
		Nmap.ne.Log("chmod 555 nping");
		os.writeBytes("chmod 555 nping\n");
		Nmap.ne.Log("chmod 555 nping");
		
		os.writeBytes("chmod 444 nmap-services\n");
		Nmap.ne.Log("chmod 444 nmap-services");
		os.writeBytes("chmod 444 nmap-mac-prefixes\n");
		Nmap.ne.Log("chmod 444 nmap-mac-prefixes");
		os.writeBytes("chmod 444 ndiff\n");
		Nmap.ne.Log("chmod 444 ndiff");*/
		
		if (NmapMain.canGetRoot)
		{
			os.writeBytes("chown root.root *\n");
			NmapMain.ne.log("chown root.root *");
		}
		os.writeBytes("chmod 555 *\n");
		NmapMain.ne.log("chmod 555 *");
		
		os.writeBytes("exit\n");
		NmapMain.ne.log("exit");
		
		StringBuilder feedback = new StringBuilder();
		String input, error;
		while ((input = in.readLine()) != null)
		{
			feedback.append(input);
		}
		while ((error = err.readLine()) != null)
		{
			feedback.append(error);
		}
		
		installationResults = feedback.toString();
		
		os.close();
		in.close();
		err.close();
		process.waitFor();
		process.destroy();
		
		if (installationResults.length() > 0)
		//if (installationResults != null && !installationResults.equals(""))
		{
			Message msg = Message.obtain();
			msg.obj = installationResults;
			msg.what = INSTALL_ERROR;
			NmapMain.handler.sendMessage(msg);
		}
	}
}
